/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file adc.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring analog-to-digital converter (ADC). 
*
*******************************************************************************
*
* provide ADC driver,complete potentiometer and thermistor test 
******************************************************************************/
#include "common.h"
#include "printf.h"
#include "adc.h"
//#include "pt60_tower.h"
/******************************************************************************
* Local function
******************************************************************************/
unsigned short ADC_GetResult( unsigned char channel);

/******************************************************************************
* Local variables
******************************************************************************/
unsigned short m_uiADC_ResultBuff[10];
unsigned char  m_ucADC_BuffCount;
/******************************************************************************
* Local function prototypes
******************************************************************************/
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_Init
   *
   * @brief initialize ADC as 12 bit mode, software trigger,enable ADC0
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void ADC_Init( void )
{
	ADC_APCTL1 = 0x01;          //disable IO on ADC CH0
	ADC_APCTL2 = 0x00;
	ADC_SC3_ADICLK = CLOCK_SOURCE_BUS_CLOCK;
	ADC_SC3_ADIV   = ADC_ADIV_DIVIDE_4;
	ADC_SC3_MODE   = ADC_MODE_12BIT;
	ADC_SC3_ADLSMP = 1;
	
	m_ucADC_BuffCount = 0;
}
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_GetResult
   *
   * @brief start a conversion and get conversion result
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
unsigned short ADC_GetResult( unsigned char channel)
{
	ADC_SC1 = channel; // initial the ADC conversion,select ADC channel
	while( !ADC_SC1_COCO );
	return ADC_R;
}


void Potentiometer_Init(void)
{
    uint8_t temp;

    if (POTENTIO_ADC_CHANNEL<8)
    {
        temp = ADC_APCTL1;
        temp |= (1<<(POTENTIO_ADC_CHANNEL%8));
        ADC_APCTL1 = temp;
   }
   else
   {
        temp = ADC_APCTL2;
        temp |= (1<<(POTENTIO_ADC_CHANNEL-8));
        ADC_APCTL2 = temp;
    }

    //ADC_APCTL2 |= 0x10;          //disable IO on ADC CH12, connected to the potentiometer
}
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: ADC_test
   *
   * @brief sample potentiometer voltage and switch LED indication according to ADC result
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void Potentiometer_Test( void )
{
    unsigned short uiADC_Value;
    unsigned long uiADC_Value_temp;

    unsigned char i = 0;

    printf("Potentiometer test: BEGINS\n");
    
    while(1)
    {
        DelayMS(1);

        uiADC_Value = ADC_GetResult(POTENTIO_ADC_CHANNEL);

        for (i=0; i<10; i++)
        {
            uiADC_Value_temp = ADC_GetResult(POTENTIO_ADC_CHANNEL);

            uiADC_Value_temp += uiADC_Value;

            uiADC_Value = (unsigned short)(uiADC_Value_temp/2);
        }


        if (uiADC_Value <= 0xFFF)
        {
            //uiADC_Value is valid, as ADC is configured as 12-bit mode
            printf("Potentiometer test: turn the Potentiometer R54 on board now\n");
            printf("Potentiometer test: pass test until the Potentiometer is turned to the max value\n");

            while (1)
            {
                //ADC measures Potentiometer value
                uiADC_Value_temp = ADC_GetResult(POTENTIO_ADC_CHANNEL);

                //calculate the average ADC value
                uiADC_Value_temp += uiADC_Value;
                uiADC_Value = (unsigned short)(uiADC_Value_temp/2);
                
                //printf ("uiADC_Value = 0x%x\n", uiADC_Value);

                //check the ADC value changes when the Potentiometer is turned
                if (uiADC_Value > (0xFFF/8)*7)
                {
                    LED0_On();
                    LED1_On();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();

                    DelayMS(500);
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_Off();
                    LED7_Off();
                    printf ("Potentiometer test: PASSED\n");
                    printf("\n");
                    return;
                }
                else if (uiADC_Value > (0xFFF/8)*6)
                {
                    LED0_Off();
                    LED1_On();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value > (0xFFF/8)*5)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value > (0xFFF/8)*4)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value > (0xFFF/8)*3)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value > (0xFFF/8)*2)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }       
                else if (uiADC_Value > (0xFFF/8)*1)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_On();
                    LED7_On();
                }
                else
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_Off();
                    LED7_On();
                }
            }
        }
        else
        {            
            //uiADC_Value is invalid
            printf ("Potentiometer test: FAILED\n");
            while (1);
        }
    }
}


void Thermistor_Init(void)
{
    uint8_t temp;

    if (THERM_ADC_CHANNEL0<8)
    {
        temp = ADC_APCTL1;
        temp |= (1<<(THERM_ADC_CHANNEL0%8));
        ADC_APCTL1 = temp;
   }
   else
   {
        temp = ADC_APCTL2;
        temp |= (1<<(THERM_ADC_CHANNEL0-8));
        ADC_APCTL2 = temp;
    }

    if (THERM_ADC_CHANNEL1<8)
    {
        temp = ADC_APCTL1;
        temp |= (1<<(THERM_ADC_CHANNEL1%8));
        ADC_APCTL1 = temp;
   }
   else
   {
        temp = ADC_APCTL2;
        temp |= (1<<(THERM_ADC_CHANNEL1-8));
        ADC_APCTL2 = temp;
    }

    //ADC_APCTL2 |= (0x1<<5);                 //disable IO on ADC CH13, connected to the Thermistor
    //ADC_APCTL2 |= (0x1<<6);                 //disable IO on ADC CH14, connected to the Thermistor
}

void Thermistor_Test( void )
{
    unsigned short uiADC_Value0, uiADC_Value1, uiADC_Value, uiADC_ValueBaseline;
    unsigned long uiADC_Value0_temp, uiADC_Value1_temp;

    unsigned char i = 0;

    printf("Thermistor test: BEGINS\n");
    
    LED7_On();

    while(1)
    {

        DelayMS(1);

        uiADC_Value0 = ADC_GetResult(THERM_ADC_CHANNEL0);
        uiADC_Value1 = ADC_GetResult(THERM_ADC_CHANNEL1);

        for (i=0; i<10; i++)
        {
            uiADC_Value0_temp = ADC_GetResult(THERM_ADC_CHANNEL0);
            uiADC_Value1_temp = ADC_GetResult(THERM_ADC_CHANNEL1);

            uiADC_Value0_temp += uiADC_Value0;
            uiADC_Value1_temp += uiADC_Value1;

            uiADC_Value0 = (unsigned short)(uiADC_Value0_temp/2);
            uiADC_Value1 = (unsigned short)(uiADC_Value1_temp/2);
        }

        uiADC_ValueBaseline = uiADC_Value0-uiADC_Value1;

        if (uiADC_ValueBaseline > 0x100)
        {
            //uiADC_ValueBaseline is valid

            printf("Thermistor test: heat the Thermistor RT1 on board now\n");
            printf("Thermistor test: pass test until the Thermistor is heated over the given value\n");
            
            while (1)
            {
                //ADC measures thermister value
                uiADC_Value0_temp = ADC_GetResult(THERM_ADC_CHANNEL0);
                uiADC_Value1_temp = ADC_GetResult(THERM_ADC_CHANNEL1);

                //calculate the average ADC value
                uiADC_Value0_temp += uiADC_Value0;
                uiADC_Value1_temp += uiADC_Value1;
                uiADC_Value0 = (unsigned short)(uiADC_Value0_temp/2);
                uiADC_Value1 = (unsigned short)(uiADC_Value1_temp/2);
                
                uiADC_Value = uiADC_Value0-uiADC_Value1;

                //printf ("uiADC_Value = 0x%x\n", uiADC_Value);

                //check the ADC value changes when the thermistor is heated
                if (uiADC_Value < uiADC_ValueBaseline -0x70)
                {
                    LED0_On();
                    LED1_On();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();

                    DelayMS(500);
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_Off();
                    LED7_Off();
                    printf ("Thermistor test: PASSED\n");
                    printf("\n");
                    return;
                }
                else if (uiADC_Value < uiADC_ValueBaseline -0x60)
                {
                    LED0_Off();
                    LED1_On();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value < uiADC_ValueBaseline -0x50)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_On();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value < uiADC_ValueBaseline -0x40)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_On();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value < uiADC_ValueBaseline -0x30)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_On();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }
                else if (uiADC_Value < uiADC_ValueBaseline -0x20)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_On();
                    LED6_On();
                    LED7_On();
                }       
                else if (uiADC_Value < uiADC_ValueBaseline -0x10)
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_On();
                    LED7_On();
                }
                else
                {
                    LED0_Off();
                    LED1_Off();
                    LED2_Off();
                    LED3_Off();
                    LED4_Off();
                    LED5_Off();
                    LED6_Off();
                    LED7_On();
                }
            }
        }
        else
        {            
            //uiADC_ValueBaseline is invalid
            printf ("Thermistor test: FAILED\n");
            while (1);
        }
    }
}
	
