/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file iic.h
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring inter-integrated circuit (IIC).
*
*******************************************************************************/
#ifndef IIC_H_
#define IIC_H_

#include "common.h"
#include "mc9s08pt60.h"

//typedef unsigned char uint8_t;
//typedef	unsigned short uint16_t;


#define i2c_DisableAck()       I2C_C1 |= I2C_C1_TXAK_MASK

#define i2c_SendAck()          I2C_SMB |= I2C_SMB_FACK_MASK; \
	                           I2C_C1 &= ~I2C_C1_TXAK_MASK     

#define i2c_RepeatedStart()    I2C_C1 |= I2C_C1_RSTA_MASK;

#define i2c_Start()            I2C_C1 |= I2C_C1_TX_MASK;\
                               I2C_C1 |= I2C_C1_MST_MASK;\
                               while(!I2C_S_BUSY)       // wait until busy

#define i2c_Stop()             I2C_C1 &= ~I2C_C1_MST_MASK;\
                               I2C_C1 &= ~I2C_C1_TX_MASK;\
                               while(I2C_S_BUSY)        // wait until not busy

#define i2c_EnterRxMode()      I2C_C1 &= ~I2C_C1_TX_MASK;\
                               I2C_C1 &= ~I2C_C1_TXAK_MASK

#define i2c_Wait()             while((I2C_S & I2C_S_IICIF_MASK)==0); \
                               I2C_S |= I2C_S_IICIF_MASK

#define i2c_write_byte(data)   I2C_D = data

#define MWSR                   0x00  /* Master write  */
#define MRSW                   0x01  /* Master read   */

extern void I2C_Init(void);
extern void I2C_Start(void);
extern void I2C_Stop(void);
extern void I2C_RepeatStart(void);
extern void I2C_Delay(void);
extern void I2C_CycleWrite(byte bout);
extern byte I2C_CycleRead(byte ack);
extern void IIC_StartTransmission (unsigned char slaveAddress, unsigned char mode);

#endif /* IIC_H_ */
