/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file irda.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief IRDA deriver,to implement IRDA communicate with ACMP 
*
******************************************************************************/
#include "common.h"
#include "Uart.h"
#include "irda.h"
#include "s08p_lite_board.h"
#include "printf.h"
#include "string.h"
#include "ftm.h"
#include "acmp.h"

extern void DelayUS(uint16_t timeUS);

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/
uint8_t u8IRDA_TxData[16] = {0x00};
uint8_t u8IRDA_RxData[16] = {0x00};
/******************************************************************************
* Local functions
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/



/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: IRDA_Init
   *
   * @brief initialize IRDA, enable ACMP ,SCI0 , 38khz as carrier
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void IRDA_Init(void)
{
    //IRDA output init: SCI0 Tx + PWM
    //SCI0: 1200bps
    Init_SCI0(BUS_CLK_HZ);
    //initialize FTM0CH0 as 38k PWM output for modulate
    FTM0CH0_Init();

    //IRDA input init: ACMP -> SCI0 Rx
    //ACMP: positive input: 2.70V DAC, negative input: IRDA received signal 2.50V/2.80V
    ACMP_Init();
    //config transceiver signal interconnections
    SYS_SOPT2_RXDFE = 1;        // receiver: ACMP output is connected to RXD0
    //the PWM will be always on the TXD0 pin.
    SYS_SOPT2_TXDME = 1;        // transmitter: FTM0CH0 PWM output AND TXD0 signal
}

void IRDA_Deinit(void)
{
    //disable SCI0 TX/RX
    SCI0_C2  = 0x0C;
    //disable FTM0Ch0 output
    FTM0_SC = 0x00; // bus clock divide by 2

    //disable DAC
    ACMP_C1 &= (~ACMP_C1_DACEN_MASK);
    
    //dsiable transceiver signal inter-connections
    SYS_SOPT2_RXDFE = 0;        // receiver: ACMP output is connected to RXD0
    //the PWM will not be on the TXD0 pin.
    SYS_SOPT2_TXDME = 0;        // transmitter: FTM0CH0 PWM output AND TXD0 signal
}
void IRDA_Transmit(uint8_t u8TxData)
{
	volatile uint8_t dummy;
	
	//SCI0_C2  |= 0x08;  // enable TX
	
	//enable the PWM/SCI_TXD
	//SYS_SOPT2_TXDME = 1;        // transmitter: FTM0CH0 PWM output AND TXD0 signal
	
	// if have invalid data in receiving buff,clear flag and discard it 
	if((SCI0_S1 & SCI0_S1_RDRF_MASK) == SCI0_S1_RDRF_MASK)
	{
		dummy = SCI0_S1;
		dummy = SCI0_D;
	}
	
	// ready for transmit a frame data and receive it from echo board and verify received data 
	TERMIO_PutChar1(u8TxData);
	
	//disable the PWM/SCI_TXD
	//SCI0_C2  &= (~0x08);               // disable TX
	//SYS_SOPT2_TXDME = 0;        // transmitter: FTM0CH0 PWM output AND TXD0 signal
	
	//return 0;
}

uint8_t IRDA_Receive(uint8_t *u8RxData)
{
    uint16_t uiCount;
    volatile uint8_t dummy;

    //SCI0_C2  = 0x04;  // disable TX , enable RX

    uiCount = 0xffff;
    while( (( SCI0_S1 & SCI0_S1_RDRF_MASK) != SCI0_S1_RDRF_MASK)&&(--uiCount));

    if( uiCount != 0 )
    {
        dummy = SCI0_S1;
        *u8RxData = SCI0_D;

        return 0;
    }
    else
    {
        //if can't receive date on SCI, delay sometime for sync
        //DelayUS(4000);

        return 1;
    }

}

void IRDA_Test(void)
{
    uint8_t i;
    uint8_t u8TestResult = 0x00;
    
    printf("IRDA test: BEGINS\n");

    printf("IRDA test: connect jumper J5 2-3\n");
    printf("IRDA test: power on another IRDA transceiver board, and place the two boards close\n");
    printf("IRDA test: pass test until the IRDA send and receive all data\n");
    printf("Transmit\t\tReceive\n");

    while (1)
    {
        for (i=0; i<8; i++)
        {
            u8IRDA_TxData[i] = i;
            u8IRDA_RxData[i] = 0x00;
            
            IRDA_Transmit(u8IRDA_TxData[i]);

            if (IRDA_Receive(&u8IRDA_RxData[i]) == 0)
            {
                printf("0x%x", u8IRDA_TxData[i]);
                printf("\t\t\t0x%x\n", u8IRDA_RxData[i]);

                //success to receive data
                if( u8IRDA_RxData[i] == u8IRDA_TxData[i])
                {
                    u8TestResult &= (~(1<<i));
                }
                else
                {
                    //the received data is wrong
                    u8TestResult |= (1<<i);
                } 
            }
            else
            {
                //fail to receive data 
                printf("0x%x", u8IRDA_TxData[i]);
                printf("\t\t\ttimeout\n");
                
                u8IRDA_RxData[i] = 0;
                u8TestResult |= (1<<i);
            }

            //delay at least one byte time to make sure the stop bit is received, then start next byte transmit
            DelayMS(4);	
        }

        if (u8TestResult == 0)
        {
            printf("IRDA test: PASSED\n");
            printf("\n");

            IRDA_Deinit();

            return;
        }
        else
        {
            printf("IRDA test: FAILED, test result = 0x%x\n", u8TestResult);
            printf("IRDA test: start the data transceive again\n");
            printf("Transmit\t\tReceive\n");

            DelayMS(1000);	
        }
    }
}

