/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file mtim.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring modulo timer (MTIM). 
*
******************************************************************************/
#include "common.h"
#include "mtim.h"

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Constants and macros
******************************************************************************/

/******************************************************************************
* Local types
******************************************************************************/

/******************************************************************************
* Local function prototypes
******************************************************************************/

/******************************************************************************
* Local variables
******************************************************************************/

/******************************************************************************
* Local functions
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: MTIM0_Init
   *
   * @brief initialize MTIM0
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void MTIM0_Init( void )
{
    //MTIM interrupt interval: CLKS/PS/MOD=16M/256/100=625
    MTIM0_CLK_CLKS = 0x00;            //clock source: bus clock
    MTIM0_CLK_PS = 0x8;                 // clock prescale: 256
    MTIM0_MOD	= 99;		    //modulo value: 100
    MTIM0_SC_TRST = 1;		    // RST count
    MTIM0_SC_TOIE = 1;                  // enable TOF interrupt
    MTIM0_SC_TSTP = 1;                  // stop count
}

void MTIM0_Start(void)
{
    MTIM0_SC_TSTP   = 0;      // start count
}

void MTIM0_Stop(void)
{
    MTIM0_SC_TSTP   = 1;      // stop count
}
/*****************************************************************************//*!
   +FUNCTION----------------------------------------------------------------
   * @function name: MTIM1_Init
   *
   * @brief initialize MTIM1
   *        
   * @param  none 
   *
   * @return none
   *
   * @ Pass/ Fail criteria: none
   *****************************************************************************/
void MTIM1_Init(void)
{
    //MTIM overflow interval: CLKS/PS/MOD = 31.25K/2/15=1.042KHz=~1ms
    MTIM1_CLK_CLKS = 0x01;           // clock source: ICSFFCLK 31.25KHz/2
    MTIM1_CLK_PS     = 0x0;            // prescale: no prescale
    MTIM1_MOD	  = 14;               //modulo value: 15
    MTIM1_SC_TRST   = 1;		   // RST count
    MTIM1_SC_TOIE   = 1;               // enable TOF interrupt
    MTIM1_SC_TSTP   = 1;               // stop count
}

void MTIM1_Start(void)
{
    MTIM1_SC_TSTP   = 0;      // start count
}

void MTIM1_Stop(void)
{
    MTIM1_SC_TSTP   = 1;      // stop count
}


