/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file tsi.h
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief providing APIs for configuring touch sense input (TSI). 
*
******************************************************************************/
#ifndef TSI_H_
#define TSI_H_

#include "s08p_lite_board.h"

//TSI channel ID definition
typedef enum
{
    TSI_0 = 0,                          //responding to TSI channel 0
    TSI_1,
    TSI_2,
    TSI_3,
    TSI_4,
    TSI_5,
    TSI_6,
    TSI_7,
    TSI_8,
    TSI_9,
    TSI_10,
    TSI_11,
    TSI_12,
    TSI_13,
    TSI_14,
    TSI_15,
    MAX_TSI_NUM,
} TSI_CHANNEL_ID_E;

typedef struct
{
    unsigned char TSI_channel;
    unsigned char touchkey_baseline_delta;
    unsigned short touchkey_touch_delta;
    unsigned char touchkey_state;
    unsigned char touchkey_touch_debounce;
    unsigned short touchkey_release_debounce;
    unsigned short touchkey_touch_threshold;
    unsigned short touchkey_baseline;
    unsigned short touchkey_current_value;
} TOUCHKEY_TSI_MAPPING_T;   




typedef enum
{
    TSI_CHARGE_CURRENT_500nA = 0,
    TSI_CHARGE_CURRENT_1uA = 1,
    TSI_CHARGE_CURRENT_2uA = 2,
    TSI_CHARGE_CURRENT_4uA = 3,
    TSI_CHARGE_CURRENT_8uA = 4,
    TSI_CHARGE_CURRENT_16uA = 5,
    TSI_CHARGE_CURRENT_32uA = 6,
    TSI_CHARGE_CURRENT_64uA = 7,
} TSI_CHARGE_CURRENT_VALUE_E;

typedef enum
{
    TSI_SOFTWARE_TRIGGER_SCAN = 0,
    TSI_HARDWARE_TRIGGER_SCAN = 1,
} TSI_SCAN_TRIGGER_MODE_E;


//key state indicates long term status changes
typedef enum
{
    TOUCHKEY_STATE_IDLE = 0,         //TSI is not touched
    TOUCHKEY_STATE_TOUCHED,          //TSI is in touched status
    TOUCHKEY_STATE_STICK,            //TSI has been touched for a long time
    TOUCHKEY_STATE_RELEASED,         //TSI is released
    TOUCHKEY_STATE_ABNORMAL          //TSI is in abnormal status
} TOUCHKEY_STATE_E;

//key event indicates momentary actions
typedef enum
{
	TOUCHKEY_EVENT_IDLE = 0,         				//key is not touched
	TOUCHKEY_EVENT_TOUCH,            				//key is touched for the first time
	TOUCHKEY_EVENT_TOUCHED,          				//key has been touched
	TOUCHKEY_EVENT_STICK,            				//key has been touched for a long time
	TOUCHKEY_EVENT_RELEASE,          				//key is released for the first time
	TOUCHKEY_EVENT_RELEASED,          				//key has been released
	TOUCHKEY_EVENT_ABNORMAL,          				//key is in abnormal status
} TOUCHKEY_EVENT_E;

//group key, which is supposed to be connected to 1/2/3/4 TSI channel.
extern TOUCHKEY_TSI_MAPPING_T TouchKey_TSI[];         //defined by user in TSI_config.c 

#define MAX_TOUCH_KEY_NUM					(sizeof(TouchKey_TSI)/sizeof(TOUCHKEY_TSI_MAPPING_T)-1)


extern void TSI_Init(void);
extern void TSI_Test(void);
extern unsigned char TSI_Task(unsigned char *current_touchkey_id);

#endif /* TSI_H_ */

