/******************************************************************************
*
* Copyright (c) 2013 - 2016, Freescale Semiconductor, Inc.
* Copyright 2016-2017 NXP
* All rights reserved.
*
***************************************************************************//*!
*
* @file w25q32.c
*
* @author nxa07754
*
* @version 0.0.1
*
* @date Oct 20, 2017
*
* @brief SPI Flash. 
*
******************************************************************************/
#include "common.h"
#include "spi.h"
#include "printf.h"


uint8_t u8FlashReadBuffer[16];
uint8_t u8FlashWriteBuffer[16];


//read w25q32 unique ID
unsigned char flash_read_uniqueid(void)
{
    unsigned char IDcode;

    SPI1_CS_LOW();
    SPI1_WriteReadByte(0x4b, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0xff, &IDcode);
    SPI1_CS_HIGH(); 

    return IDcode;                                                                                         
}

//read w25q32 unique ID
unsigned char flash_read_deviceid(void)
{
    unsigned char deviceID;

    SPI1_CS_LOW();
    SPI1_WriteReadByte(0x90, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0x00, NULL);
    SPI1_WriteReadByte(0xff, &deviceID);
    SPI1_CS_HIGH(); 

    return deviceID;                                                                                         
}


//read w25q32 status
unsigned char flash_read_status(void)
{
    unsigned char  status;

    SPI1_CS_LOW();
    SPI1_WriteReadByte(0x05, NULL);                                                
    SPI1_WriteReadByte(0xff, &status);                                        
    SPI1_CS_HIGH();
    return status;
}


//enable flash write
void flash_write_enable(void)
{
    SPI1_CS_LOW();
    SPI1_WriteReadByte(0x06, NULL);                                                                                       
    SPI1_CS_HIGH();
}

//disable flash write
void flash_write_disable(void)
{
    SPI1_CS_LOW();
    SPI1_WriteReadByte(0x04, NULL);                                                                                       
    SPI1_CS_HIGH();
}

//read w25q32 data
uint8_t flash_read_data(uint32_t u32FlashAddr, uint8_t *u8Buffer, uint8_t u32Bytes)
{
    uint8_t u8Temp;
    uint32_t i;

    if (u32Bytes == 0)
    {
        return 0xFF;
    }

    while(1)                                                                                                                
    {  
        //Check the status register. If it's busy , wait until it's free
        u8Temp = flash_read_status( );                                                                
        u8Temp &= 0x01;                                                                                        
        if(u8Temp == 0x00)   
        {
            break;        
        }
        for(i=0; i<10; i++);                                                
    }

    SPI1_CS_LOW();                                                                                                             
    SPI1_WriteReadByte(0x03, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0xFF0000)>>16, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0x00FF00) >> 8, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0x0000FF), NULL);
    
    for (i=0; i<u32Bytes; i++)
    {
        SPI1_WriteReadByte(0xff, &u8Buffer[i]);
    }
    SPI1_CS_HIGH();                                                                                                            

    return 0;
}

//W25Q32 erase sector per 4K byte
uint8_t flash_sector_erase(uint32_t u32FlashAddr)
{
         flash_write_enable();                                                                                                    /* Write enable                 */
         
         SPI1_CS_LOW();                                                                                                             
         SPI1_WriteReadByte(0x20, NULL);
         SPI1_WriteReadByte((u32FlashAddr & 0xFF0000) >> 16, NULL);
         SPI1_WriteReadByte((u32FlashAddr & 0x00FF00) >> 8, NULL);
         SPI1_WriteReadByte(u32FlashAddr & 0x0000FF, NULL);
          SPI1_CS_HIGH();                                                                                                            

         while (flash_read_status() & 0x01 != 0x00);  

         return 0;
}

//W25Q32 write sector per 4K byte
uint8_t flash_write_sector(uint32_t u32FlashAddr, uint8_t *u8Buffer, uint32_t u32Bytes)
{
    uint32_t i;

    if (u32Bytes == 0)
    {
    return 0xFF;
    }

    flash_write_enable();                                                                                                    /* Write enable                 */


    SPI1_CS_LOW();                                                                                                             /* P0.2--0,CS = 0 ??SPI Flash */
    SPI1_WriteReadByte(0x02, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0xFF0000) >> 16, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0x00FF00) >> 8, NULL);
    SPI1_WriteReadByte((u32FlashAddr & 0x0000FF), NULL);
    for (i=0; i<u32Bytes; i++)
    {
        SPI1_WriteReadByte(u8Buffer[i], NULL);
    }
    SPI1_CS_HIGH();                                                                                                            /* P0.2--1,CS = 1 ??SPI Flash */

    while ((flash_read_status() & 0x01) != 0x00);

    return 0;
}


void FlashW25Q32_Init(void)
{
    //Init_SPI;
    SPI1_Init();
}

void FlashW25Q32_Test(void)
{
    uint8_t i;
    uint8_t deviceID;
    uint8_t returnResult;
    uint16_t testResult = 0;

    printf("FlashW25Q32 test: BEGINS\n");

   deviceID = flash_read_deviceid();
   printf("FlashW25Q32 test: read device ID is 0x%x\n", deviceID);

    for (i=0; i<16; i++)
    {
        u8FlashReadBuffer[i] = 0;
        u8FlashWriteBuffer[i] = i;
    }

    printf("FlashW25Q32 test: erase flash sector 0\n");
    printf("FlashW25Q32 test: after erase, contents in sector 0 should be all 0xFF\n");
    returnResult = flash_sector_erase(0);

    returnResult = flash_read_data(0, u8FlashReadBuffer, 16);
    printf("FlashW25Q32 test: read flash sector 0:\n");
    for (i=0; i<16; i++)
    {
        printf(" 0x%x", u8FlashReadBuffer[i]);

        if (u8FlashReadBuffer[i] != 0xFF)
        {
            testResult |= 1<i;
        }
    }
    printf("\n");

    if (testResult != 0)
    {
        printf("FlashW25Q32 test: FAILED, test result = 0x%x\n", testResult);
        printf("\n");

        return;
    }
    
    testResult = 0;

    printf("FlashW25Q32 test: write flash sector 0:\n");
    printf("FlashW25Q32 test: after write,  contents in sector 0 should equal to what's written\n");

    for (i=0; i<16; i++)
    {
        printf(" 0x%x", u8FlashWriteBuffer[i]);
    }
    printf("\n");
  
    returnResult = flash_write_sector(0, u8FlashWriteBuffer, 16);

    returnResult = flash_read_data(0, u8FlashReadBuffer, 16);
    printf("FlashW25Q32 test: read flash sector 0:\n");
    for (i=0; i<16; i++)
    {
        printf(" 0x%x", u8FlashReadBuffer[i]);

        if (u8FlashReadBuffer[i] != u8FlashWriteBuffer[i])
        {
            testResult |= 1<i;
        }
    }
    printf("\n");

    
    if (testResult != 0)
    {
        printf("FlashW25Q32 test: FAILED, test result = 0x%x\n", testResult);
        printf("\n");
    }
    else
    {
        printf("FlashW25Q32 test: PASSED\n");
        printf("\n");

    }
    
    return;
}


