#include "s08p_lite_board.h"
#include "common.h"
#include "mtim.h"
#include "printf.h"
#include "tsi.h"
#include "wdog.h"
#include "adc.h"
#include "iic.h"
#include "uart.h"
#include "ics.h"
#include "mtim.h"
#include "ftm.h"
#include "iic.h"

/******************************************************************************
* Global functions
******************************************************************************/
void Button_Init(void);
void configure_LEDs_for_GPIO(void);
void configure_LEDs_for_PWM ();

extern uint8_t g_delay_1ms_done; 
extern uint8_t button2IsPressedFlag, button3IsPressedFlag;

#define MAX_PWM_VALUE 0

void delay()
{
    uint8_t n;
    
    for(n=0;n<50;n++) 
    {
        asm(nop);
    }
}

void DelayMS(uint16_t timeMS)
{
    uint16_t  delay_given_ms = 0;
    g_delay_1ms_done = 0;

    MTIM1_Start();
    
    while (delay_given_ms < timeMS)
    {
        while (!g_delay_1ms_done);
        g_delay_1ms_done = 0;
        delay_given_ms++;
    }

    MTIM1_Stop();
}


void BOARD_InitDebugConsole(void)
{ 
	Init_SCI2(BUS_CLK_HZ); 
}

void Sys_Init(void)
{
    /* Disable the watchdog timer */
    /* WARNINGIN: DO NOT single step this function call, otherwise
    * will cause watch dog reset
    */
    wdog_disable();	
    /*
    /////////////////////////////////////////////////////////////////////////////////////////
    // Peripheral Init
    // --------------------------------------------------------------------------------------
    /////////////////////////////////////////////////////////////////////////////////////////
    */

    /* Initialize the Pass/Fail counts to 0 */
    //global_pass_count = 0;
    //global_fail_count = 0;



    /* output bus clock if enabled */
#if defined(OUTPUT_BUSCLK)
    //SYS_SOPT3_CLKOE = 1;
    SYS_SOPT3 |= SYS_SOPT3_CLKOE_MASK;
#endif

#if defined(USE_FEE)  
    // Initialize system clock to 16MHz with external 8M crystal
    FEI_to_FEE();
#endif  



    /* enable stop,*/
    SYS_SOPT1 = 0x4d;
    /* Configure SCI for printing */
                       
    BOARD_InitDebugConsole();
    

#if	defined(PRINT_SYS_LOG)
    print_sys_log();
#endif



    /* _asm CLI */ /* enable interrupts */
}

void LED_ConfigAsGPIO(void)
{
	// configure the pin as output
	LED0_Init();
	LED1_Init();
	LED2_Init();
	LED3_Init();
	LED4_Init();
	LED5_Init();
	LED6_Init();
	LED7_Init();	

}

void BOARD_InitLED(void)
{
	
    LED_ConfigAsGPIO();
}

void BOARD_InitPins(void)
{
	BOARD_InitLED();
	
	
}

void LED_Init(void)
{
    // 8 LEDs are connected to PTE7, PTG0/1/2/3, PTH2/6/7
    
    // configure the pin as GPIO output
    LED0_Init();
    LED1_Init();
    LED2_Init();
    LED3_Init();
    LED4_Init();
    LED5_Init();
    LED6_Init();
    LED7_Init();
    
    LED0_Off();
    LED1_Off();
    LED2_Off();
    LED3_Off();
    LED4_Off();
    LED5_Off();
    LED6_Off();
    LED7_Off();
}

void LED_Test(void)
{
    LED0_On();
    DelayMS(100);
    LED1_On();
    DelayMS(100);
    LED2_On();
    DelayMS(100);
    LED3_On();    
    DelayMS(100);
    LED4_On();
    DelayMS(100);
    LED5_On();
    DelayMS(100);
    LED6_On();    
    DelayMS(100);
    LED7_On();   
    DelayMS(100);

    LED7_Off();
    DelayMS(100);
    LED6_Off();
    DelayMS(100);
    LED5_Off();
    DelayMS(100);
    LED4_Off(); 
    DelayMS(100);
    LED3_Off();
    DelayMS(100);
    LED2_Off();
    DelayMS(100);
    LED1_Off();
    DelayMS(100);
    LED0_Off();
    DelayMS(100);

#if 0
    LED0_On();
    LED1_On();
    LED2_On();
    LED3_On();    
    LED4_On();
    LED5_On();
    LED6_On();    
    LED7_On();  
#endif
}
/*****************************************************************************//*!
+FUNCTION----------------------------------------------------------------
* @function name: Button_Init
*
* @brief initialize button switches.
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void Button_Init(void)	
{	
    // 2 mechanical buttons are connected to PTD4/5

    KBI1_SC_KBIE = 0;       // disable the KBI interrupt
    KBI1_SC_KBMOD = 0;      //KBI detects edges only
    KBI1_ES &= (~KBI1_ES_KBEDG4_MASK) & (~KBI1_ES_KBEDG5_MASK);     //select the KBI polarity: falling edge
    //PORT_PTDPE = PORT_PTDPE_PTDPE4_MASK | PORT_PTDPE_PTDPE5_MASK;	// enable internal pullup
    KBI1_PE = KBI1_PE_KBIPE4_MASK | KBI1_PE_KBIPE5_MASK;                    // enable KBI pin
    KBI1_SC_KBACK = 1;	    //clear any false interrupt
    KBI1_SC_KBIE  = 1;      //enable the KBI interrupt
}

void Button_Test(void)
{

    printf("Button test: BEGINS\n");
    
    button2IsPressedFlag = 0;
    button3IsPressedFlag = 0;

    printf("Button test: press button 2 or 3 on board now\n");
    printf("Button test: pass test until button 2, 3 are pressed\n");
    
    while (1)
    {
        //waiting for both button 2, 3 are pressed
        if ((button2IsPressedFlag) && (button3IsPressedFlag))
        {
            printf("Button test: PASSED\n");
            printf("\n");
            return;
        }
    }
}

void DbgConsole_Init(void)
{
    // OSBDM serial outputs are connected to SCI2 PTD6/7

    Init_SCI2(BUS_CLK_HZ);   // init SCI2
}

void DAC5571_Init(void)
{
    // DAC5571 is connected to I2C PTA3/2

    I2C_Init();
}

void DAC5571WriteData(unsigned char u8Data)
{
    /* send data to slave */
    IIC_StartTransmission(DAC5571_I2C_ADDRESS, MWSR);
    i2c_Wait();

    /* Send I2C address */
    I2C_D = u8Data>>4;
    i2c_Wait();

    /* Send data */
    I2C_D = u8Data<<4;
    i2c_Wait();

    i2c_Stop();

    delay();
}

void DAC5571_Test(void)
{
    unsigned short uiADC_Value;
    unsigned long uiADC_Value_temp;

    unsigned char i;
    unsigned char dacOutputValue;
    unsigned char dacOutputValueMin, dacOutputValueMax;
    uint8_t temp;

    //initialize adc channel to measure the dac output
    if (DAC5571_ADC_CHANNEL<8)
    {
        temp = ADC_APCTL1;
        temp |= (1<<DAC5571_ADC_CHANNEL);
        ADC_APCTL1 = temp;
   }
   else
   {
        temp = ADC_APCTL2;
        temp |= (1<<(DAC5571_ADC_CHANNEL-8));
        ADC_APCTL2 = temp;
    }

    printf("DAC5571 test: BEGINS\n");

    printf("DAC5571 test: connect jumper J15 2-3 and J6 2-3\n");
    printf("DAC5571 test: the DAC5571 output from 0 to 255 automatically now\n");
    printf("DAC5571 test: pass test until the DAC5571 output the max value\n");

    
    
    for(dacOutputValue=0; dacOutputValue<255; dacOutputValue++)
    {
        DAC5571WriteData(dacOutputValue);
        DelayMS(10);

        uiADC_Value = ADC_GetResult(DAC5571_ADC_CHANNEL);

        for (i=0; i<10; i++)
        {
            uiADC_Value_temp = ADC_GetResult(DAC5571_ADC_CHANNEL);

            uiADC_Value_temp += uiADC_Value;

            uiADC_Value = (unsigned short)(uiADC_Value_temp/2);
        }

        //8-bit DAC, 12-bit ADC, so the mapping ratio = 4096/256 = 16
        uiADC_Value = uiADC_Value /16;
        
        dacOutputValueMin = (dacOutputValue<10)? 0:(dacOutputValue-10);
        dacOutputValueMax = (dacOutputValue>245)?255:(dacOutputValue+10);
        
        if ((uiADC_Value>=dacOutputValueMin) && (uiADC_Value <= dacOutputValueMax))
        {
            //uiADC_Value is valid, as ADC is configured as 12-bit mode
            //check the ADC value changes when the DAC5571 output is changed
            if (uiADC_Value > (0xFF/8)*7)
            {
                LED0_On();
                LED1_On();
                LED2_On();
                LED3_On();
                LED4_On();
                LED5_On();
                LED6_On();
                LED7_On();

                DelayMS(500);
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_Off();
                LED4_Off();
                LED5_Off();
                LED6_Off();
                LED7_Off();
                printf ("DAC5571 test: PASSED\n");
                printf("\n");
                
                return;
            }
            else if (uiADC_Value > (0xFF/8)*6)
            {
                LED0_Off();
                LED1_On();
                LED2_On();
                LED3_On();
                LED4_On();
                LED5_On();
                LED6_On();
                LED7_On();
            }
            else if (uiADC_Value > (0xFF/8)*5)
            {
                LED0_Off();
                LED1_Off();
                LED2_On();
                LED3_On();
                LED4_On();
                LED5_On();
                LED6_On();
                LED7_On();
            }
            else if (uiADC_Value > (0xFF/8)*4)
            {
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_On();
                LED4_On();
                LED5_On();
                LED6_On();
                LED7_On();
            }
            else if (uiADC_Value > (0xFF/8)*3)
            {
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_Off();
                LED4_On();
                LED5_On();
                LED6_On();
                LED7_On();
            }
            else if (uiADC_Value > (0xFF/8)*2)
            {
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_Off();
                LED4_Off();
                LED5_On();
                LED6_On();
                LED7_On();
            }       
            else if (uiADC_Value > (0xFF/8)*1)
            {
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_Off();
                LED4_Off();
                LED5_Off();
                LED6_On();
                LED7_On();
            }
            else
            {
                LED0_Off();
                LED1_Off();
                LED2_Off();
                LED3_Off();
                LED4_Off();
                LED5_Off();
                LED6_Off();
                LED7_On();
            }
        }
        else
        {            
            //uiADC_Value is invalid
            printf ("DAC5571 test: FAILED\n");
            printf ("DAC5571 test: stop in DAC5571 test\n");
            while (1);
        }
    }
}

void Touchpad_Init(void)
{
    // 2 touch pads are connected to PTB0/TSI2, PTF3/TSI5
    
    //TSI_Init();
}

void AnalogInput_Init(void)
{
    //init potentiometer, thermistor
    //potentiometer is connected to ADC: PTF4/ADP12
    // thermistor is connected to ADC: PTF5/ADP13, PTF6/ADP14
    
    //ADC_Init();
}

void AnalogCompare_Init(void)
{
    // CMP is connected to ACMP: PTA1/ACMP1
    // compare source can be selected from potentiometer or DAC5571

    //CMP_Init();
}

void Buzzer_Init(void)
{
    // buzzer is connected to FTM: PTC4/FTM1CH0
    //buzzer
    FTM1_Init();
}

void Buzzer_On(void)
{
    FTM1_Start();
}

void Buzzer_Off(void)
{
    FTM1_Stop();
}

void Buzzer_Beep(void)
{
    Buzzer_On();
    DelayMS(10);
    Buzzer_Off();
}

void Buzzer_Test(void)
{
    Buzzer_On();
    DelayMS(50);
    Buzzer_Off();
}
/************************************************************************
*       configure_LEDs_for_PWM - Set up TPM1 and TPM2                   *
*************************************************************************
; Since the LEDs are connected to the Timer/Pulse Width Modulator modules (TPMs),
; we can configure the TPMs to generate an edge-aligned PWM signal. 
*/
void configure_LEDs_for_PWM (void)
{
    /* Configure TPM1 to generate PWM signals */
	// always high level

	
    FTM1_SC = 0;                                 /* Stop the timer */ 

    /* Configure duty cycles of each channel
     
       NB: We choose a timer frequency = fbus / 16, so a period of 500 (MAX_PWM_VALUE)
       means that an LED will flash
       
            8 x 10E6 / (500 * 32) = 500 flashes/second 
            
       ...i.e. too fast to be detected by the human eye
    
    */
    //FTM1_MOD = MAX_PWM_VALUE; 
    
    /* Configure TPMxCnSC for each channel as follows
        CHnF    = 0  Not used
        CHnIE   = 0  No interrupt
        MSnB:A  = 10 Edge-aligned PWM
        ELSnB:A = 01 Active-low pulse
    */   
    FTM1_C0SC = 0x24;       
    FTM1_C1SC = 0x24;                                               
    

    /* Repeat for TPM2 */
    FTM2_SC = 0;      
    FTM2_MOD = MAX_PWM_VALUE;
     
    FTM2_C0SC = 0x24;       
    FTM2_C1SC = 0x24;       
    FTM2_C2SC = 0x24;       
    FTM2_C3SC = 0x24;    
    FTM2_C4SC = 0x24;       
    FTM2_C5SC = 0x24; 
    
    FTM2_C0V = 100;  
    FTM2_C1V = 100;  
    FTM2_C2V = 100;  
    FTM2_C3V = 100;  
    FTM2_C4V = 100;  
    FTM2_C5V = 100;  
    FTM1_C0V = 100; 
    FTM1_C1V = 100;
    
	PTD0_Init();
	PTD0_Set();
    
    FTM1_SC  = 0x0D;                  /* Restart the timer: CLKS = 01 (Use BUSCLK), PS=101 (Divide-by-32) */
    FTM2_SC  = 0x0D;    
    

}

/************************************************************************
*       configure_LEDs_for_GPIO - Stop TPMs from driving LEDs           *
*************************************************************************
; Modifies the TPM channel configurations so that the TPMs no longer drive
; the LEDs. Control over the LEDs reverts to the GPIO method
*/
void configure_LEDs_for_GPIO(void)
{
	// disable the FTM
	FTM1_SC = 0x0;
	FTM2_SC = 0x0;
	
	// ELSnB:A = 00 : Pin reverts to GPIO 
    FTM2_C0SC = 0x00;       
    FTM2_C1SC = 0x00;       
    FTM2_C2SC = 0x00;       
    FTM2_C3SC = 0x00;
    FTM2_C4SC = 0x00;       
    FTM2_C5SC = 0x00;
    
    FTM1_C0SC = 0x00;       
    FTM1_C1SC = 0x00;
	
	// configure the pin as output
	LED1_Init();
	LED2_Init();
	LED3_Init();
	LED4_Init();
	LED5_Init();
	LED6_Init();
	LED7_Init();
	
	
	// always high level
	PTD0_Init();
	PTD0_Set();
}

/************************************************************************
*       leds_all_on - Turn all LEDs on                                  *
*************************************************************************/
void leds_all_on (void)
{
    configure_LEDs_for_GPIO ();
    PORT_PTCD &= ~0x3F;
    PORT_PTBD &= ~0x30;
}

/************************************************************************
*       leds_all_off - Turn all LEDs off                                *
*************************************************************************/
void leds_all_off (void)
{
    configure_LEDs_for_GPIO ();
    PTD0_Clear();
}

/************************************************************************
*       leds_cross - Turn on LEDs to form a "x" cross                   *
*************************************************************************
*/
void leds_cross(void)
{
    configure_LEDs_for_GPIO ();
    
    /* Set active LEDs */
    LED2_On();   //D6
    LED4_On();   //D8
    LED6_On();   //D10

    LED1_Off();   //D6
    LED3_Off();   //D8
    LED5_Off();   //D10
    LED7_Off();   //D12      
}

/************************************************************************
*       leds_plus - Turn on LEDs to form a "+" plus sign                *
*************************************************************************
*/
void leds_plus(void)
{
    configure_LEDs_for_GPIO ();
    
    /* Set active spokes */
    LED1_On();   //D6
    LED3_On();   //D8
    LED5_On();   //D10
    LED7_On();   //D12   
    
    LED2_Off();   //D6
    LED4_Off();   //D8
    LED6_Off();   //D10
    
}
/*
 * end here
 */
