#include <hidef.h>                  /* for EnableInterrupts macro */
#include "derivative.h"             /* include peripheral declarations */
#include "common.h"
#include "mtim.h"
#include "printf.h"
#include "tsi.h"
#include "wdog.h"
#include "adc.h"
#include "s08p_lite_board.h"
#include "string.h"
#include "math.h"
#include "irda.h"
#include "rtc.h"
#include "iic.h"
#include "spi.h"
#include "ftm.h"
#include "w25q32.h"

#if 0
byte last_sysmod_reg;                   /* Last value of SYSMOD_REG */
byte last_f_status_reg;                 /* Last value of F_STATUS_REG */
byte last_transient_src_reg;            /* Last value of TRANSIENT_SRC_REG */ byte last_pl_status_reg;                /* Last value of PL_STATUS_REG */
byte last_pulse_src_reg;                /* Last value of PULSE_SRC_REG */
byte last_ff_mt_src_1_reg;              /* Last value of FF_MT_SRC_1_REG */
byte last_xyz_values [6];               /* Last values of X, Y and Z */ 
#endif

unsigned char buttonStates = 0;

unsigned short g_delay_ms = 0;
uint8_t g_delay_1ms_done = 0;


#define Button2_Is_Pressed()	!(buttonStates & (1<<4))
#define Button3_Is_Pressed()	!(buttonStates & (1<<5))

uint8_t button2IsPressedFlag = 0;
uint8_t button3IsPressedFlag = 0;

uint8_t touchpad1IsTouchedFlag = 0;
uint8_t touchpad2IsTouchedFlag = 0;

void DelayUS(unsigned short timeUS)
{
	unsigned short i, j;
	for(i = 0; i < timeUS; i++)
	{
		for(j = 0; j < 16; j++)	/* delay 1 us */
		{
			asm(nop);
		}
	}
}

/*****************************************************************************//*!
+FUNCTION----------------------------------------------------------------
* @function name: KEY1_Processing
*
* @brief handles touch pad 1 on board
*        
* @param  none 
*
* @return none
*
* @ Pass/ Fail criteria: none
*****************************************************************************/

void KEY1_Processing( void )
{	

}


/************************************************************************
*       FEI default mode configuration                                                                             *
*************************************************************************/
void ICS_FEI(void)
{
	ICS_C1 = 0x04;    // Internal reference clock selected
	ICS_C2 = 0x20;    // Divides selected clock by 2 (reset default)
//	ICS_C3 = 0x80;    // internal reference clock trim value
	
	while(!ICS_S_LOCK);  // wait clock to lock 
}


    
/************************************************************************
*       main - Program main entry point                                                                         *
*************************************************************************/
void main(void) 
{
    uint8_t i = 0;
    
    uint8_t u8IRDATxCnt = 0;
    
    volatile uint8_t w25q32_id = 0;
 //   volatile unsigned char dummy;
 //   unsigned short uiCount;
    
    //Initialize system including clocks
    //Watchdog: disabled
    //FEE: BUS clock 16MHz
    //Serial terminal: SCI2 baudrate 115200
    Sys_Init();
    EnableInterrupts;

    printf("\n-- S08-LITE FAT BEGINS --\n");
    printf("\n");
    
    //Initialize MTIM1, 1ms interrupt interval, used for 1ms time delay
    MTIM1_Init();
    
    //Initialize FTM FTM1CH0 used for buzzer
    Buzzer_Init();
    //Test Buzzer, turn on/off
    Buzzer_Test();

    //Initialize GPIO used for LED
    LED_Init();
    //Test LED0-7, every LED will be turn on/off one by one
    LED_Test();
    Buzzer_Beep();
    
    //Initialize ADC, 12bit mode
    ADC_Init();
    
    //Initialize ADC ADP13/14 user for thermistor measurement
    Thermistor_Init();
    Thermistor_Test();
    Buzzer_Beep();

    //Initialize ADC ADP12 user for potentiometer measurement
    Potentiometer_Init();
    Potentiometer_Test();
    Buzzer_Beep();
    
    //Initialize I2C PTA2/3 used for DAC5571 write/read
    DAC5571_Init();
    DAC5571_Test();
    Buzzer_Beep();

    //Initialize SPI1 PTD0/1/2/3 used for flash W25Q32
    FlashW25Q32_Init();
    FlashW25Q32_Test();
    Buzzer_Beep();
    
    //Initialize GPIO PTD4/5 used for KBI SW2/SW3
    Button_Init();
    Button_Test();
    Buzzer_Beep();
     
    //Initialize TSI ch2, ch15 used for touch pad E1, E2
    TSI_Init();
    TSI_Test();
    Buzzer_Beep();
    
    //Initialize RTC, 10ms interrupt interval, used for trigger TSI software scan
    RTC_Init();

    //Initialize SCI0 Tx/FTM0Ch0 used for IRDA transmit, SCI0 Rx/ACMPIn0 used for IRDA receive
    IRDA_Init();
    IRDA_Test();
    Buzzer_Beep();
    
    printf("\n-- S08-LITE FAT ENDS --\n");
    DelayMS(1000);
    Buzzer_Beep();

    //Initialize MTIM0, 625Hz interrupt interval, used for toggle LED
    MTIM0_Init();
    MTIM0_Start();
    
    while (1);
    
}

uint16_t mtim0_count = 0;
interrupt VectorNumber_Vmtim0 void  Mtim0_ISR(void)
{
    if(MTIM0_SC_TOF)  // clear the flag
    {
        MTIM0_SC_TOF = 0;		
    }

    mtim0_count++;
    if (mtim0_count > 625)
    {
        //toggle LED every 1 second
        LED4_Toggle();
        mtim0_count = 0;
    }
}

//MTIM1 is used for delay 1 milliseconds
interrupt VectorNumber_Vmtim1 void  Mtim1_ISR(void)
{
    if(MTIM1_SC_TOF)
    {
        // clear the flag
        MTIM1_SC_TOF = 0;	
    }
    g_delay_1ms_done = 1;
}

//RTC 10ms interrupt: software trigger TSI scan
interrupt VectorNumber_Vrtc void  RTC_ISR(void)
{
    uint8_t u8TouchKeyEvent = 0;
    uint8_t u8TouchKeyId = 0;

	if(RTC_SC1_RTIF)  // clear the flag
	{
            RTC_SC1_RTIF = 1;	
	}
			
        u8TouchKeyEvent = TSI_Task(&u8TouchKeyId);

        if (u8TouchKeyEvent == TOUCHKEY_EVENT_TOUCH)
        {
            
        
            if (u8TouchKeyId == 0)
            {
                touchpad1IsTouchedFlag = 1;
                LED1_On();
            }
            else
            {
                touchpad2IsTouchedFlag = 1;
                LED6_On();
            }
#if 0
            u8FlashWriteBuffer[0] ++;

            flash_sector_erase(0);
            //flash_read_data(0, u8FlashReadBuffer, 16);
            flash_write_sector(0, u8FlashWriteBuffer, 16);
            //flash_read_data(0, u8FlashReadBuffer, 16);

#endif          
        }
        else if (u8TouchKeyEvent == TOUCHKEY_EVENT_RELEASE)
        {
            if (u8TouchKeyId == 0)
            {
                LED1_Off();
            }
            else
            {
                LED6_Off();
            }
            
        }

}

interrupt  VectorNumber_Vtsi  void  TSI_ISR(void)
{	
//	unsigned short uiDelta;
	if(TSI_CS0_EOSF)   // clear the flag
	{
		TSI_CS0_EOSF = 1;
	}   		

}

interrupt  VectorNumber_Vkbi1 void  Button_ISR(void)
{
    KBI1_SC_KBACK = 1;                       //clear the KBI interrupt flag
    //KBI1_PE = 0;                                // NOTE: disable KBI pin function to avoid the mechanical jitters

    KBI1_PE &= (~KBI1_PE_KBIPE4_MASK) & (~KBI1_PE_KBIPE5_MASK);                    // disable KBI pin
    PORT_PTDIE |= (1<<4)|(1<<5);    //enable gpio input
    asm(nop);
    asm(nop);
    buttonStates = PORT_PTDD;         //read PORT data register to recognize which button is pushed

    if(Button2_Is_Pressed())
    {
        LED0_Toggle();
        button2IsPressedFlag = 1;
    }
    else if (Button3_Is_Pressed())
    {
        LED7_Toggle();
        button3IsPressedFlag = 1;
    }
    else
    {
        //
    }

    // restore KBI pin function after PORT data register
    KBI1_PE = KBI1_PE_KBIPE4_MASK | KBI1_PE_KBIPE5_MASK; /* enable KBI pin */
    KBI1_SC_KBACK = 1;	
}

interrupt  VectorNumber_Vacmp void  ACMP_Int_ISR(void)
{
    ACMP_CS_ACF = 0;

    if (ACMP_CS_ACO)
    {
        LED0_Off();
    }
    else
    {
        LED0_On();
    }

}


